/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * * 
 * Use is subject to license terms.
 ******************************************************************/
package isac.gui;

import isac.bridge.CalcTree;
import isac.gui.treetable.CalculationModel;
import isac.gui.treetable.CustomTreeCellRenderer;
import isac.gui.util.CalcModelHierarchy;
import isac.gui.util.CalcModelNode;
import isac.util.CalcChangedEvent;
import isac.util.Formalization;
import isac.util.formulae.Assumptions;
import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcHeadCompoundID;
import isac.util.formulae.CalcFormula;
import isac.util.interfaces.ICalcIterator;
import isac.util.interfaces.IToUser;
import isac.util.interfaces.IToWorksheet;
import isac.util.tactics.Tactic;
import isac.util.users.IUserSettings;
import isac.wsdialog.DGuide;
import isac.wsdialog.DialogProtocolException;
import isac.wsdialog.IUIElement;
import isac.wsdialog.IUserAction;
import isac.wsdialog.UserAction;
import isac.wsdialog.UserActionOnCalcElement;
import isac.wsdialog.UserActionOnCalcHeadCompoundID;
import isac.wsdialog.UserActionOnIterator;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.Locale;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.tree.DefaultTreeCellEditor;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeSelectionModel;

import org.apache.log4j.Logger;

/**
 * <code>Worheet</code> is the component that is responsible for the
 * interaction with the user while doing a calculation. Furthermore it acts as a
 * visual represantiation of mathematical formulas in a calculation tree, that
 * is beeing executed by the isac knowledge interpreter.
 * 
 * The <code>Worksheet</code> is the view of the
 * <code>WorksheetDialogGuide</code>.
 * 
 * The data, that the <code>WorksheetDialogGuide</code> delivers, is
 * representated in the hierarchical structur <code>JTree</code>.
 * 
 * @author Mario Hochreiter
 * @version 0.1 15/09/2003
 */
public class Worksheet extends JPanel implements ActionListener, /* IToUser, */
CellEditorListener, Serializable {

	private MouseAdapter ml;

	private JMenuItem applicableTacticsMenuItem_;

	private JMenuItem tactic_applied_menuitem_;

	private JMenuItem assumptionsMenuItem_;

	private JMenuItem intermediate_steps_menuitem_;

	private JPopupMenu popupMenu;

	private JPopupMenu applicableTacticsMenu_;

	private Point mouse_point_;

	private Component mouse_component_;

	private Vector applicableTactics_;

	static Logger logger = Logger.getLogger(Worksheet.class.getName());

	protected IToUser worksheet_rmi_1_;

	protected IToWorksheet worksheet_rmi_2_;

	/**
	 * variable that says if the calculation has ended just to inhibit <Next>
	 * <Auto>
	 */
	protected boolean calculationEnded;

	// formula which will be edited if the dialog allows it
	protected CalcFormula edited_formula_;

	/** Model for the JTree */
	protected TreeModel model;

	/**
	 * LK+WN0505 remove: only used for getting DialogIterators; DGuide knows
	 * CalcTree, get Iterator from there
	 */
	protected CalcTree calc_tree_;

	protected CalcModelHierarchy calc_;

	/** Used to represent the model. */
	protected JTree tree;

	/** Dialog which is responsible for the communication with the kernel */
	protected DGuide dialog_;

	/** Button for doing the next step in calculation */
	//protected JButton nextStep;
	/** Calculate whole example (without single stepping) */
	//protected JButton autoCalc;
	/**
	 * true if it is allowed to call tryMatch/tryRefine, false otherwiese WN0504
	 * should be due to UserSettings
	 * 
	 * WN050421 kept due to isCallProblemAllowed used in initMouseListener
	 * WN+LK0504: made superfluous: remove ?
	 */
	protected boolean call_problem_allowed_;

	/**
	 * true if in single stepping mode WN0504 should be due to UserSettings
	 * WN+LK0504: made superfluous: remove
	 */
	protected boolean single_stepping_;

	protected JPanel button_panel_;

	/** Panel which is shows the calcHead if forced */
	protected CalcHeadPanel chp_;

	/** Panle which shows the tactic if forced */
	protected TacticPanel tp;

	/** submit calcHead and start calculation */
	protected JButton start_solving_;

	/** check calc head */
	protected JButton checkCalcHead;

	/** complete calc head */
	protected JButton compCalcHead;

	/**
	 * the position in the CalcTree any changes (by User + mathEngine) are
	 * applied to
	 */
	protected ICalcIterator active_formula_;

	protected CalcHead ch_;

	//LK 04.11.18
	protected int current_row_;

	/**
	 * Create a <code>Worksheet</code> (and display the Specification if
	 * requested by the viewStyle according to UserSettings). The given
	 * <code>DGuide</code> is connected to the <code>Worksheet</code>.
	 * 
	 * @param l
	 *            Locale that is used.
	 * @param dialogGuide
	 *            The dialog guide that is connected to the Worksheet
	 * @param f
	 *            Formalization necessary to start a new calculation
	 * @param source
	 *            Where was the calculation started...
	 * @see ...{isac.wsdialog.DGuide#STARTFROM_*}
	 */
	public Worksheet(final Locale l, DGuide dialogGuide, Formalization f,
			int source) {
		// LK 04.11.18 (1 line)
		current_row_ = 0;
		super.setLayout(new BorderLayout());
		single_stepping_ = true;//WN050420 remove
		call_problem_allowed_ = true;//WN050420 remove
		dialog_ = dialogGuide;
		try {
			worksheet_rmi_1_ = new WorksheetRMI(this);
			worksheet_rmi_2_ = new WorksheetRMI(this);
			dialog_.addDataChangeListener(worksheet_rmi_1_);
			dialog_.registerUIControlListener(worksheet_rmi_2_);

			calc_tree_ = dialog_.startCalculation(0, f, source);
			//WN050421-AK: Worksheet should _ONLY_ know about DialogIterator
			//and use notifyUserAction for Calculation-related sutff.
			// But THIS system-generated call still requires calc_tree_ !!
			this.ch_ = startSpecifying(l);

			this.replaceCalcHeadPanelByWorksheet();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			//    } catch (DialogProtocolException e) {
			//		logger.error(e);
		}
		//outputText = new JTextArea(10, 50);//WN050421???
		//this.add(new JScrollPane(outputText),
		// BorderLayout.CENTER);//WN050421???
	}

	/**
	 * pop the CalcHeadPanel or go immediately to Worksheet according to
	 * UserSettings returned from the DGuide (via VIEWSTYLE_*x)
	 */
	private CalcHead startSpecifying(Locale l) {
		CalcHead ch = null;//WN0504 take ch_ ???
		try {
			ch = dialog_.startSpecifying();
		} catch (RemoteException e) {
			e.printStackTrace();
		}

		chp_ = new CalcHeadPanel(l, ch/* , ch.getViewStyle() */);//#####
		this.buildCalcHeadPanelButtons();//#####
		super.add(chp_, BorderLayout.NORTH);//#####

		switch (ch.getViewStyle()) {
		case CalcHead.VIEWSTYLE_SINGLELINE:
			break;
		case CalcHead.VIEWSTYLE_FULL:
			//#####
			this.showCalcHeadPane();
			break;
		}
		return ch;
	}

	private void buildCalcHeadPanelButtons() {
		button_panel_ = new JPanel();
		//TODO localize!!!
		start_solving_ = new JButton("start Solving");
		compCalcHead = new JButton("complete CalcHead");
		checkCalcHead = new JButton("check CalcHead");
		start_solving_.addActionListener(this);
		compCalcHead.addActionListener(this);
		checkCalcHead.addActionListener(this);
		button_panel_.add(checkCalcHead);
		button_panel_.add(compCalcHead);
		button_panel_.add(start_solving_);
		super.add(button_panel_, BorderLayout.SOUTH);
	}

	//  private void buildButtons() {
	//    this.remove(buttonPanel);
	//    buttonPanel = new JPanel();
	//    //TODO localize!!!
	//    nextStep = new JButton("Next step");
	//	  autoCalc = new JButton("Autocalc");
	//    nextStep.addActionListener(this);
	//  	autoCalc.addActionListener(this);
	//    buttonPanel.add(nextStep);
	//    buttonPanel.add(autoCalc);
	//    this.add(buttonPanel, BorderLayout.SOUTH);
	//  }

	/**
	 * buildMenu to be called by clicking right mousebutton
	 */
	protected void buildMenu() {
		popupMenu = new JPopupMenu();
		applicableTacticsMenuItem_ = new JMenuItem("applicable tactics");
		applicableTacticsMenuItem_.addActionListener(this);
		tactic_applied_menuitem_ = new JMenuItem("tactic applied");
		tactic_applied_menuitem_.addActionListener(this);
		assumptionsMenuItem_ = new JMenuItem("assumptions");
		assumptionsMenuItem_.addActionListener(this);
		intermediate_steps_menuitem_ = new JMenuItem("intermediate steps");
		intermediate_steps_menuitem_.addActionListener(this);
		popupMenu.add(applicableTacticsMenuItem_);
		popupMenu.add(tactic_applied_menuitem_);
		popupMenu.add(assumptionsMenuItem_);
		popupMenu.add(intermediate_steps_menuitem_);
		this.initMouseListener();
	}

	protected void buildApplicableTacticsMenu() {
		applicableTacticsMenu_ = new JPopupMenu();
		Iterator it = applicableTactics_.iterator();
		Integer step = new Integer(0);
		while (it.hasNext()) {
			Tactic tactic = (Tactic) it.next();
			JMenuItem item = new JMenuItem(tactic.toSMLString());
			item.setActionCommand(step.toString());
			item.addActionListener(this);
			applicableTacticsMenu_.add(item);
			int stepInt = step.intValue();
			stepInt++;
			step = new Integer(stepInt);
		}
		applicableTacticsMenu_.show(mouse_component_, mouse_point_.x,
				mouse_point_.y);
	}

	protected void initMouseListener() {
		ml = new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				logger.info("mouseEvent");
				if (e.getButton() == MouseEvent.BUTTON3
						&& !isCallProblemAllowed()
						&& (tree.getSelectionPath() != null)) {
					mouse_point_ = e.getPoint();
					mouse_component_ = e.getComponent();
					popupMenu.show(e.getComponent(), e.getX(), e.getY());
				}
			}
		};
		tree.addMouseListener(ml);
	}

	private void showTacticPanel() {
		if (tp == null) {
			tp = new TacticPanel();
			super.add(tp, BorderLayout.EAST);
		}
	}

	/* LK 04.09.02 */
	public boolean isCalculationEnded() {
		return this.calculationEnded;
	}

	/**
	 * TreeView; attachs the Worksheet to the CalcHeadPanel
	 */
	public void setTreeTableModel() {
		calc_ = new CalcModelHierarchy();
		model = new CalculationModel(calc_, calc_tree_.iterator());
		tree = new JTree(model);
		tree.setEditable(true);
		tree.setCellRenderer(new CustomTreeCellRenderer());
		tree.setCellEditor(new DefaultTreeCellEditor(tree,
				new DefaultTreeCellRenderer()));
		//LK 04.11.17: (1 line)
		tree.getCellEditor().addCellEditorListener(this);
		tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setRootVisible(false);
		tree.setShowsRootHandles(true);
		KeyStroke ks = KeyStroke.getKeyStroke(KeyEvent.VK_A,
				KeyEvent.CTRL_DOWN_MASK);
		tree.getInputMap().put(ks, "none");
		super.add(new JScrollPane(tree), BorderLayout.CENTER);
	}

	/**
	 * Do matching (see UC 2.5.6).
	 * 
	 * @param problemID
	 *            unique id for a CalcHead
	 */
	public void tryMatch(CalcHeadCompoundID problemID) {
		try {
			// dialog.tryMatch(ch, problemID);
			dialog_.notifyUserAction(new UserActionOnCalcHeadCompoundID(
					IUIElement.UI_SPECIFY_TRY_MATCH, problemID));
			logger.debug("CalcHead after tryMatch: " + ch_.toSMLString());
			chp_.fillTextFieldsFromCalcHead(ch_);
			this.showCalcHeadPane();
		} catch (RemoteException e) {
			e.printStackTrace();
			//    } catch (NotInSpecificationPhaseException e) {
			//      e.printStackTrace();
		} catch (DialogProtocolException e) {
			logger.error(e.getMessage(), e);
		}
	}

	/**
	 * Refine a problem (see UC 2.5.7).
	 * 
	 * @param problemID
	 *            unique id for a CalcHead
	 */
	// method at the moment only gets CalcHead but does not
	// mark the refined problem in the problem hierarchy
	public void tryRefine(CalcHeadCompoundID problemID) {
		try {
			// dialog.tryRefine(ch, problemID);
			dialog_.notifyUserAction(new UserActionOnCalcHeadCompoundID(
					IUIElement.UI_SPECIFY_TRY_REFINE, problemID));
			chp_.fillTextFieldsFromCalcHead(ch_);
			this.showCalcHeadPane();
		} catch (DialogProtocolException e) {
			logger.error(e);
		} catch (RemoteException e) {
			logger.error(e);
		}
	}

	/**
	 * Show all applicable Tactics that can be used for the next calculation
	 * step (UC 2.5.11).
	 * 
	 * @param scope
	 *            The filter parameter selects the scope of search for
	 *            applicable tactics.
	 */
	public void showApplicableTactics(int scope) {
		//FIXME.WN0504 use a DialogIterator !!!
		//		try {
		//			Tactic[] tactic = dialog_.fetchApplicableTactics(scope);
		//			//TODO: show tactis in a list the user can select
		//		} catch (RemoteException e) {
		//			logger.error(e);
		//		}
	}

	/**
	 * Call problem is allowed if the calculation is not in solving phase.
	 */
	public boolean isCallProblemAllowed() {
		return call_problem_allowed_;
	}

	/**
	 * @return true if the calculation has started.
	 */
	public boolean isCalculationStarted() {
		return active_formula_ != null;
	}

	private void setIterator() {
		try {
			active_formula_ = calc_tree_.iterator();
			//FIXXME.WN05011 active_formula_ = dialog_.iterator(); !!!
			active_formula_.moveRoot();
		} catch (RemoteException e) {
			e.printStackTrace();
			System.exit(5);
		}
	}

	private void showCalcHeadPane() {
		super.add(chp_, BorderLayout.NORTH);
		super.validate();
	}

	/* LK 04.09.01 */
	// to be called from the windowApplication
	public void nextStep() {
		try {
			dialog_.notifyUserAction(new UserAction(IUIElement.UI_CALCULATE_1));
			//      singleStepping = true;
			//      Tactic propTactic;
			//      try {
			//        if (it_ == null)
			//          this.setIterator();
			//        propTactic = dialog.fetchProposedTactic();
			//        //FIXME:WN: vvvv ...endCalculation
			//        if (propTactic != null) {
			//          dialog.setNextTactic(propTactic);
			//        } else {
			//          this.calculationEnded = true;
			//        }
			//      } catch (RemoteException e1) {
			//        e1.printStackTrace();
			//      } //try
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DialogProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/* LK 04.09.01 */
	// to be called from the windowApplication
	public void autoCalc() {
		//if (!this.calculationEnded) {
		try {
			single_stepping_ = false;
			dialog_.notifyUserAction(new UserAction(
					IUserAction.UI_CALCULATE_ALL));
			for (int i = 0; i < tree.getRowCount(); i++) {
				tree.expandRow(i);
			}
			this.calculationEnded = true;
		} catch (RemoteException e1) {
			logger.error("actionPerformed", e1.detail);
		} catch (DialogProtocolException e1) {
			logger.error("actionPerformed");
		} //try
		//}//if
	}

	/**
	 * replace the CalcHeadPanel by the Worksheet
	 *  
	 */
	private void replaceCalcHeadPanelByWorksheet() {
		this.setIterator();
		this.setTreeTableModel();
		super.remove(button_panel_);//at the bottom of CalcHeadPanel
		this.buildMenu();
		super.remove(chp_);//from top of Worksheet
		this.showTacticPanel();//WN050421???
		this.validate();//WN050421???
	}

	/**
	 * 
	 * 
	 * Description of the method in HTML format, if a link can be given in the
	 * running text do this with {@link
	 * fully.qualified.Classname#methodName(fully.qualified.Paramclass)}
	 * 
	 * @param <paramname>
	 *            <paramdescription>
	 * @return <return value>
	 * @exception <exception>
	 *                <description when it is thrown>
	 * @see <fully.qualified.Classname#methodName(param-classes)>
	 */
	public void startSolving() {
		try {
			//    dialog.newCalculation(ch);
			chp_.fillCalcHeadFromTextFields(ch_);
			logger.fatal("WS->DG: UI_SPECIFY_TO_SOLVE");
			if (dialog_.notifyUserAction(new UserAction(
					IUserAction.UI_SPECIFY_TO_SOLVE))) {
				replaceCalcHeadPanelByWorksheet();
			} else {
				JOptionPane.showMessageDialog(this,
						"Please enter a correct CalcHead",
						"CalcHead not correct", JOptionPane.WARNING_MESSAGE);
			}
		} catch (RemoteException e1) {
			logger.error("actionPerformed", e1.detail);
		} catch (DialogProtocolException e2) {
			logger.error("actionPerformed", e2);
		}
	}

	public void compCalcHead() {
		try {
			chp_.fillCalcHeadFromTextFields(ch_);
			dialog_.notifyUserAction(new UserAction(
					IUserAction.UI_SPECIFY_COMPLETE_CALCHEAD));
			chp_.fillTextFieldsFromCalcHead(ch_);
		} catch (Exception e1) {
			logger.error(e1);
		}
	}

	public void checkCalcHead() {
		try {
			logger.debug(dialog_);
			dialog_.notifyUserAction(new UserAction(
					IUserAction.UI_SPECIFY_CHECK_CALCHEAD));
			chp_.fillTextFieldsFromCalcHead(ch_);
		} catch (Exception e1) {
			logger.error(e1);
		}
	}

	/**
	 * shows the applicable tactics, if applied on the activeFormula a
	 * applicable tactic can be chosen and is applied to this formula
	 */
	public void applicableTactics() {
		try {
			CalcModelNode node = (CalcModelNode) tree.getSelectionPath()
					.getLastPathComponent();
			ICalcIterator it = node.getValue();
			applicableTactics_ = it.getApplicableTactics(0);
			buildApplicableTacticsMenu();
			//WN0504 DialogIterator checks wrt. Usersettings etc., thus NOT...
			//      dialog.notifyUserAction(new UserAction(
			//          IUserAction.UI_SOLVE_GET_APPLICABLE_TACTICS));
		} catch (Exception e1) {
			logger.error(e1);
		}
	}

	/**
	 * shows the Tactic that was applied to this formula
	 *  
	 */
	public void tacticApplied() {
		try {
			CalcModelNode node = (CalcModelNode) tree.getSelectionPath()
					.getLastPathComponent();
			node.tacticString = ((ICalcIterator) node.getValue()).getTactic()
					.getName();
			tree.repaint();
		} catch (Exception e1) {
			logger.error(e1);
		}
	}

	public void assumptions() {
		try {
			System.out.println("assumtions");
		} catch (Exception e1) {
			logger.error(e1);
		}
	}

	/**
	 * WN050511->LK? explanation please
	 */
	public void intermediateSteps() {
		try {
			CalcModelNode node = (CalcModelNode) tree.getSelectionPath()
					.getLastPathComponent();
			ICalcIterator it = node.getValue();
			calc_tree_.intermediateSteps(it);
		} catch (Exception e1) {
			logger.error(e1);
		}
	}

	//------------- Implementation of ActionListener ----------------
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == start_solving_) {
			startSolving();
		} else if (e.getSource() == compCalcHead) {
			compCalcHead();
		} else if (e.getSource() == checkCalcHead) {
			checkCalcHead();
		} else if (e.getSource() == applicableTacticsMenuItem_) {
			applicableTactics();
		} else if (e.getSource() == tactic_applied_menuitem_) {
			tacticApplied();
		} else if (e.getSource() == assumptionsMenuItem_) {
			assumptions();
		} else if (e.getSource() == intermediate_steps_menuitem_) {
			intermediateSteps();
		} else if (e.getActionCommand() != null
				&& !e.getActionCommand().equals("")) {
			int pos = Integer.parseInt(e.getActionCommand());
			Tactic tactic = (Tactic) applicableTactics_.get(pos);
			try {
				dialog_.notifyUserAction(new UserActionOnCalcElement(
						IUserAction.UI_SOLVE_SET_NEXT_TACTIC, tactic));
			} catch (RemoteException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (DialogProtocolException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		} else {
			//unexpected event source
			logger.error("unexpected event source " + e.getSource());
		}
	}

	public void calcChanged(CalcChangedEvent event) {
		logger.debug("single Stepping: " + single_stepping_);
		logger.debug(model);
		logger.debug("calc" + calc_);
		logger.debug(model);
		((CalculationModel) model).calcChanged(event);
		for (int i = 0; i < tree.getRowCount(); i++) {//WN050428 ???+DG filters
			tree.expandRow(i);
		}
		tree.setEditable(true);
		//TOODO.WN0504DI assumes on CalculationModel#calcChanged !
		//  if (DialogIterator)event.getLastGeneratedFormula().onCalcHead()
		//     then this.startSpecifying
	}

	/**
	 * if the user has canceled the editing, the worksheet should inform the
	 * dialogGuide
	 * 
	 * @see javax.swing.event.CellEditorListener#editingCanceled(javax.swing.event.ChangeEvent)
	 */
	public void editingCanceled(ChangeEvent e) {
		// TODO Auto-generated method stub

	}

	/**
	 * if the user has changed the value of a node, it should be altered
	 * 
	 * @see javax.swing.event.CellEditorListener#editingStopped(javax.swing.event.ChangeEvent)
	 */
	public void editingStopped(ChangeEvent e) {
		Object o = tree.getCellEditor().getCellEditorValue();
		//WN050511->LK? just delete
		//TODO: LK 04.11.17: temporary solution:

		//    if (!((CalculationModel)model).editNode(current_node,
		// (String)tree.getCellEditor().getCellEditorValue())) {
		//      JOptionPane.showMessageDialog(this, "unable to find a way to this
		// step",
		//                                      "Error!", JOptionPane.ERROR_MESSAGE);
		//    }
		//tree.getModel().valueForPathChanged(tree.getSelectionPath()), o);
		//LK 04.11.17: end of temporary solution:
		try {
			CalcModelNode current_node = (CalcModelNode) tree
					.getSelectionPath().getLastPathComponent();
			active_formula_ = current_node.getValue();
			if (active_formula_ == null) { // append formula
				//appends after the currently active formula
				edited_formula_ = new CalcFormula();
				edited_formula_.setText((String) o);
				logger.fatal(" WS->DG: UI_SOLVE_APPEND_USER_FORMULA");
				dialog_.notifyUserAction(new UserAction(
						IUserAction.UI_SOLVE_APPEND_USER_FORMULA));
			} else { //replace formula
				logger.fatal(" WS->DG: UI_SOLVE_MOVE_ACTIVE_FORMULA");
				dialog_.notifyUserAction(new UserActionOnIterator(
						IUserAction.UI_SOLVE_MOVE_ACTIVE_FORMULA,
						active_formula_));
				edited_formula_ = new CalcFormula();
				edited_formula_.setText((String) o);
				edited_formula_.setPosition(active_formula_.getPosition());
				logger.fatal(" WS->DG: UI_SOLVE_EDIT_ACTIVE_FORMULA");
				dialog_.notifyUserAction(new UserAction(
						IUserAction.UI_SOLVE_EDIT_ACTIVE_FORMULA));
			}
		} catch (Exception e1) {
			e1.printStackTrace();
		}
		System.out.println(o);
	}

	/**
	 * @param action
	 */
	public boolean doUIAction(IUserAction action) {
		int actionID = action.getActionID();
		switch (actionID) {
		case IUIElement.UI_DO_EDIT_FORMULA:
			try {
				logger.fatal(" WS->DG: UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE");
				dialog_.notifyUserAction(new UserActionOnCalcElement(
						IUserAction.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE,
						edited_formula_));
			} catch (RemoteException e) {
				e.printStackTrace();
			} catch (DialogProtocolException e) {
				e.printStackTrace();
			}
			return true;
		case IUIElement.UI_DO_APPEND_FORMULA:
			try {
				logger.fatal(" WS->DG: UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE");
				dialog_.notifyUserAction(new UserActionOnCalcElement(
						IUserAction.UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE,
						edited_formula_));
			} catch (RemoteException e) {
				e.printStackTrace();
			} catch (DialogProtocolException e) {
				e.printStackTrace();
			}
		default:
			return false;
		}

	}

}