/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.wsdialog;

import java.rmi.Remote;
import java.rmi.RemoteException;

import isac.bridge.CalcTree;
import isac.util.*;
import isac.util.formulae.*;
import isac.util.interfaces.*;

/**
 * @author Alan Krempler
 *
 */
/**
 * @author Alan Krempler
 * 
 * The interface of the Dialog Guide as seen from the Presentation Layer
 *  
 */
public interface DGuide extends IUIElement, /*IToCalc,*/ Remote {

	/**
	 * To express that the user started a calculation from scratch
	 */
	public static final int STARTFROM_NEW = 1;

	/**
	 * To express that the user started a calculation from scratch WN050415 this
	 * decision is triggered by the CalcHead.viewstyle
	 */
	//public static final int STARTFROM_EMPTY_CALCHEAD = 2;
	/**
	 * To express that the user started a calculation from the example
	 * collection
	 */
	public static final int STARTFROM_EXAMPLE = 3;

	/**
	 * To express that the user started a calculation from the hierarchy of
	 * problems
	 */
	public static final int STARTFROM_PROBLEM = 4;

	public static final int STARTFROM_METHOD = 5;

	/**
	 * The Dialog is not doing anything at the moment
	 */
	public static final int DIALOGPHASE_IDLE = 10;

	/**
	 * The Dialog guides the user through the specification of a problem;
	 * WN050420 in this phase the CalcHeadPanel is presented to the User; the
	 * phase is entered iff:
	 * 
	 * (in_SPECIFY_1) the Worksheet does CalcIterator#getFormula while
	 * CalcIterator.onCalcHead _AND_ the DialogGuide agrees due to UserSettings
	 * 
	 * (in_SPECIFY_2) the Worksheets activeFormula is on a headline of a
	 * CalcHead and the User selects the menuitem 'CalcHead'
	 * 
	 * This phase is finished iff:
	 * 
	 * (out_SPECIFY_1) the User closes the CalcHeadPanel (which requires a
	 * 'complete' CalcHead)
	 * 
	 * (out_SPECIFY_2) the Worksheet does CalcTree#replaceFormula at a Position
	 * the active CalcHeadPanel depends on (this allows to escape from an
	 * 'incomplete' subproblem) TODO
	 */
	public static final int DIALOGPHASE_SPECIFY = 11;

	/**
	 * The Dialog guides the user to the solution of a problem
	 * 
	 * WN050420 this phase is entered iff:
	 * 
	 * (in_SOLVE_0) a new Worksheet is opened; this is done immediately followed
	 * by DIALOGPHASE_SPECIFY case (I1) thus commonly starting a calculation
	 * with the DIALOGPHASE_SPECIFY
	 * 
	 * (in_SOLVE_1)=(out_SPECIFY_1)
	 * 
	 * (in_SOLVE_2)=(out_SPECIFY_2)
	 * 
	 * This phase is finished iff:
	 * 
	 * (out_SOLVE_1)=(in_SPECIFY_1)
	 * 
	 * (out_SOLVE_2)=(in_SPECIFY_2) TODO
	 */
	public static final int DIALOGPHASE_SOLVE = 12;

	/**
	 * The Dialog guides the user through the specification of a subproblem
	 * 
	 * @deprecated WN0504 since there is no related description
	 */
	public static final int DIALOGPHASE_SPECIFY_SUBPROBLEM = 3;

	/**
	 * Start the a calculation by providing initial values to the Dialog On
	 * success, the Dialog is in DIALOGPHASE_SOLVE
	 * 
	 * @param user_id
	 *            The user
	 * @param f
	 *            Formalization from the example collection or null
	 * @param started_from
	 *            STARTFROM_*
	 * @return @throws
	 *         RemoteException
	 * 
	 * @see isac.util.Formalization
	 */
	public CalcTree startCalculation(int user_id, Formalization f,
			int started_from/* , int requested_calchead_view */)
			throws RemoteException;

	/**
	 * Enter the solving phase of the calculation. StartSpecifying must be
	 * called first Only possible with a complete and correct calcHead
	 * 
	 * @param calcHead
	 *            CalcHead for this calculation
	 * @return CalcTree represents the calculation on the java side
	 */
	public void startSolving(/* CalcHead calcHead */) throws Exception;

	/**
	 * Start the specification phase by providing initial values to the Dialog
	 * On success, the Dialog is in DIALOGPHASE_SPECIFY
	 * 
	 * @see isac.wsdialog.DGuide#DIALOGPHASE_SPECIFY *
	 * @throws Exception
	 *             because working within DialogGuide
	 */
	//CalcHead startSpecifying(CalcHead c) throws RemoteException;//WN050421
	// OLD
	public CalcHead startSpecifying() throws RemoteException;//WN050421 NEW

	/**
	 * Start the solving phase. On success, the Dialog is in DIALOGPHASE_SOLVE
	 * 
	 * @throws Exception
	 *             because working within DialogGuide
	 */
	//public void startSolving() throws Exception;//WN050421 NEW
	//public void startSolving(CalcHead c) throws Exception;//WN050421 OLD
	/**
	 * Register the UI to be controlled by the Dialog. There is only one such
	 * UI, any already-registered UI will be detached.
	 * 
	 * @param listener
	 *            UI which is giong to listen to the Dialog
	 * @return true on success, flase othewise
	 * @throws RemoteException
	 */
	public boolean registerUIControlListener(IToWorksheet listener)
			throws RemoteException;

	/**
	 * Notify the Dialog about events or requests triggered by the user
	 * 
	 * @param action
	 *            What's new from the UI
	 * @return true, if the event is being processed, false if processing is
	 *         denied or impossible
	 * @throws RemoteException
	 * @throws DialogProtocolException
	 */
	public boolean notifyUserAction(IUserAction action) throws RemoteException,
			DialogProtocolException;

	/**
	 * Register a new listener who wants to be notified of changes in the data.
	 * The listener has to implement the IToUser interface.
	 * 
	 * @param listener
	 *            New listener to be registered.
	 * @return false on success, true otherwise.
	 * 
	 * WN050429 copied from IToCalc upon removing 'DGuide implements IToCalc'
	 * redesign interfaces ?
	 */
	public boolean addDataChangeListener(IToUser listener)
			throws RemoteException;

//	/**
//	 * Show all applicable Tactics that can be used for the next calculation
//	 * step (UC 2.5.11).
//	 * 
//	 * @param scope
//	 *            The filter parameter selects the scope of search for
//	 *            applicable tactics. FIXXXME.WN050429 added upon removing
//	 *            'DGuide implements IToCalc' a hack until this is replaced by
//	 *            notifyUserAction, too
//	 */
//	public void showApplicableTactics(int scope) throws RemoteException;
}